package GeneralTools;

use strict;
use warnings;
use Exporter;
use Socket;

@GeneralTools::EXPORT_OK = qw#reverse_lookup#;
@GeneralTools::ISA = ('Exporter');

my $lookup_cache = {};

return (1);

sub reverse_lookup
{
	my $val = shift;
	
	return unless ($$val =~ m/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/);

	print $/, __FILE__, ": Trying reverse lookup for $$val" if ($::DEBUG & 0x4000);
	
	if (defined ($lookup_cache->{$$val}))
	{
		my $host = $lookup_cache->{$$val};
		$$val = $host if ($host);
		print ': Cached (', $host ? $host : '-failed-', ')' if ($::DEBUG & 0x4000);
		return;
	}
	
	my $iaddr = inet_aton ($$val);
	if (!defined ($iaddr))
	{
		print ": Failed (not a valid IP: $$val)" if ($::DEBUG & 0x4000);
		return;
	}
	
	my $host = gethostbyaddr ($iaddr, AF_INET ());

	if ($host)
	{
		$lookup_cache->{$$val} = $host;
		$$val = $host;
		print ": Success ($host)" if ($::DEBUG & 0x4000);
	}
	else
	{
		$lookup_cache->{$$val} = '';
		print ': Failed (unknown)' if ($::DEBUG & 0x4000);
	}

	return;
}
