package Parser;

use strict;
use warnings;
use vars qw(%extra %datafields);

use Exporter;
use WebserverTools qw(%month_numbers);
use GeneralTools qw(reverse_lookup);
use Config qw#get_config#;

@Parser::EXPORT_OK = qw(parse extra %datafields);

@Parser::ISA = ('Exporter');

# FIXME pass date and time in seconds to properly format and sort.

$extra{'total'} = 0;
$extra{'days'} = {};

%datafields = (
    host	=> 'key:host',
    user	=> 'key',
    date	=> 'key',
    hour	=> 'key',
    domain	=> 'key:host',
    tld		=> 'key',
    file	=> 'key',
    status	=> 'key',
    bytes	=> 'amount:bytes',
    requests	=> 'amount:number'
);

# This needs to be done at runtime, since Data uses Setup which relies on
# %datafields to be defined  -octo
require Data;
import Data qw#store#;

print $/, __FILE__, q(: Using Apache's access-log format) if ($::DEBUG);

my $rev_lup = 0;
{
	my $rl = get_config ('reverse_lookup');
	if ($rl =~ m/true|yes/i) { $rev_lup = 1; }
}

return (1);

sub parse
{
	my $line = shift or return undef;
	if ($line =~ /^(\S+)\s(\S+)\s(\S+)\s\[(.+?)\]\s"(.+?)"\s(\d+)\s(.+)$/)
	{
		my ($host, $ident, $user, $date, $request, $status, $bytes) =
			($1, $2, $3, $4, $5, $6, $7);

		my ($day, $month, $year, $hour, $minute, $second) =
			$date =~ m#(\d\d)/(\w{3})/(\d{4}):(\d\d):(\d\d):(\d\d)#;
		
		my $n_date = $year . '-' . $month_numbers{$month} . '-' . $day;

		my ($method, $file, $params) =
			$request =~ m#(\S+) ([^\?]+)\??(\S*)#
			or return (0);
		
#		my $attack = 0;
#		if (lc ($file) eq '/default.ida') { ++$extra{'codered'}; $attack = 1; }
#		elsif (lc ($file) eq '/scripts/root.exe') { ++$extra{'nimda'}; $attack = 1; }
#		elsif ($file =~ /root\.exe|winnt/i) { $attack = 1; }
#		if ($attack and !$::config->{'inc_attacks'}) { return 'attack'; }

		if (($user ne '-') and ($status >= 400) and ($status < 500))
		{
			$user = 'invalid';
		}

		if ($user eq '-') { $user = 'unknown'; }
		if ($bytes eq '-') { $bytes = 0; }

		reverse_lookup (\$host) if ($rev_lup);
		my ($domain, $tld);
		if ($host =~ m/([^\.]+)\.([a-z]{2,})$/i)
		{
			$tld = lc ($2);
			$domain = lc ($1) . ".$tld";
		}
		else
		{
			$domain = 'UNRESOLVED';
			$tld = 'UNRESOLVED';
		}

		++$extra{'total'};
		if (!defined $extra{'days'}->{$n_date}) { $extra{'days'}->{$n_date} = 1; }

		my %combined = (
					'host'		=>	$host,
					'user'		=>	$user,
					'date'		=>	$n_date,
					'hour'		=>	$hour,
					'domain'	=>	$domain,
					'tld'		=>	$tld,
					'file'		=>	$file,
					'status'	=>	$status,
					'bytes'		=>	$bytes,
					'requests'	=>	1
				);
		store (\%combined);
	}
	elsif ($::DEBUG)
	{
		chomp ($line);
		print STDERR "\nUnable to parse: '$line'";
	}
}

sub extra
{
	my ($average, $days) = (0, 0);
	
	$days = scalar keys %{$extra{'days'}};

	return (0) unless ($days);
	
	$average = int (($extra{'total'} * 10) / $days) / 10;
	$::EXTRA->{'Total requests'} = $extra{'total'};
	$::EXTRA->{'Averange requests per day'} = $average;
	$::EXTRA->{'Reporting period'} = "$days days";
#	$::EXTRA->{'CodeRed attacks'} = $extra{'codered'};
#	$::EXTRA->{'Nimda attacks'} = $extra{'nimda'};
}
