package Report;

use strict;
use warnings;

=head1 topN.pm (report)

=head2 Generates

Generates Top-N pages for each key and for each aggregation selected.
The grouping is also accepted - then generationg topN pages for all 
groups too.

=head2 Config

Accepts selections:

Select keys or key combination for which to generate pages:

    select: key;
    select: GROUP;

Select aggregations to use as sorting value:

    select: fun(foo);

=head2 Debugging

    ::DEBUG bit 1 set (==2) - dumps loaded data and exits;
    ::DEBUG bit 4 set (==16) - verbose initialization; 

=cut

use vars qw(@groups $topcount);
use Config qw#get_config#;
use Core
qw#
	@keyfields @selections @aggregations %aggregations @sortorder
	%datatitles $reportype_title @pages $data_ref $table_refname
	$table_caption

	keysort unroll setup format_titles strfval file_idx_ref
	generate_table generate_general generate_head
#;
use Utils qw#cmp_arrays printgrouping trans#;
use Html qw#foot#;

$VERSION='topN';

# contains simple list of all groups defined
@groups=();

#### BEGIN

setup ();

our $topcount = get_config ('topcount');
$topcount ||= 16;

print $/, __FILE__, ": Generating top$topcount report." if $::DEBUG;

unless(@keyfields) {
    print STDERR "\nreport/$VERSION: FATAL: no data to report defined";
    die "\nCheck your config file\n";
    }

foreach my $groupset ( @selections ) {
    push @groups,map(@$_,@$groupset);
    }

@pages=@groups;

if( $::DEBUG&16 ) {
    print "\nreport/$VERSION: Using keys:";
    foreach my $gr (@groups) {
	print "\n\t".printgroup(@$gr);
	}
    }

# empty index is to get total amounts
Data::setup([ [[],@groups],[values %aggregations] ]);

#### END of BEGIN

sub generate
{
    my ($directory,$filename);
    if ($::DEBUG&2) { Data::dump(); return; }

    $reportype_title="Top$topcount";
    format_titles();

    #
    # I want all files to be created in single place.
    # And i ever know - why...
    #
    $directory=$main::outputdir;

    $filename=$directory."index.html";
    print STDERR "\nGeneral page..." if $::DEBUG;
    open (HTMLFILE, "> $filename") or die "when opening $filename: $!\n";
    generate_general(*HTMLFILE);
    close(HTMLFILE);

    foreach my $gr (@groups) {
	$filename=$directory.file_ref(@$gr);
	print STDERR "\nTop$topcount page for".printgroup(@$gr)."..." if $::DEBUG;
	open (HTMLFILE, "> $filename") or die "when opening $filename: $!\n";
	generate_toppage(*HTMLFILE,$gr);
	close(HTMLFILE);
	}

}

sub generate_toppage
{
    my ($FILE,$fields)=@_;
    generate_head($FILE,@$fields);

    if( @aggregations > 1 ) {
	print {$FILE} "\n<table class='index'><tr><td class='index' valign=middle border=0>";
	foreach my $cat (@aggregations) {
	    my $link = '#'.table_name(@{$aggregations{$cat}});
	    my $title = $datatitles{$cat};
	    print {$FILE} "\n<a class='index' href='$link'>".
			  "[\&nbsp;$title\&nbsp;]".
			  "</a>";
	    }
	print {$FILE} "\n</table>";
	}

    print {$FILE} "\n<hr>";

    $table_refname = \&table_name;
    my @keys = unroll($fields);
    foreach my $aggr (@aggregations) {
	my @filtered = sort { amount($b,$aggr) <=> amount($a,$aggr); } @keys;
	# splice do the right thing if $#keys < $topcount
	@filtered = splice(@filtered,0,$topcount);
	generate_table($FILE,{},[$aggr],\@filtered,$fields);
	print {$FILE} '<br>';
	}

    print {$FILE} foot();
}

#### subs to handle @pages for references generating

sub file_ref { return file_idx_ref(@_).".html"; }

sub page_title 
{
    return trans('by /%s/, /', @_);
}

sub page_header
{
    return "Top$topcount ".page_title(@_);
}

sub table_name
{
    my %key=@_;
    return join('',map("$_($key{$_})",grep($_=~/[A-Z]/,keys %key)));
}

=head1 AUTHOR

QMax E<lt>qmax@mail.ruE<gt>

=cut

1;
